﻿/*************** 
 * Irinfo Test *
 ***************/

import { core, data, sound, util, visual, hardware } from './lib/psychojs-2023.1.2.js';
const { PsychoJS } = core;
const { TrialHandler, MultiStairHandler } = data;
const { Scheduler } = util;
//some handy aliases as in the psychopy scripts;
const { abs, sin, cos, PI: pi, sqrt } = Math;
const { round } = util;


// store info about the experiment session:
let expName = 'IrInfo';  // from the Builder filename that created this script
let expInfo = {
    'participant': `${util.pad(Number.parseFloat(util.randint(0, 999999)).toFixed(0), 6)}`,
    'session': '001',
};

// Start code blocks for 'Before Experiment'
// init psychoJS:
const psychoJS = new PsychoJS({
  debug: true
});

// open window:
psychoJS.openWindow({
  fullscr: true,
  color: new util.Color([0,0,0]),
  units: 'height',
  waitBlanking: true
});
// schedule the experiment:
psychoJS.schedule(psychoJS.gui.DlgFromDict({
  dictionary: expInfo,
  title: expName
}));

const flowScheduler = new Scheduler(psychoJS);
const dialogCancelScheduler = new Scheduler(psychoJS);
psychoJS.scheduleCondition(function() { return (psychoJS.gui.dialogComponent.button === 'OK'); }, flowScheduler, dialogCancelScheduler);

// flowScheduler gets run if the participants presses OK
flowScheduler.add(updateInfo); // add timeStamp
flowScheduler.add(experimentInit);
flowScheduler.add(trial1RoutineBegin());
flowScheduler.add(trial1RoutineEachFrame());
flowScheduler.add(trial1RoutineEnd());
flowScheduler.add(quitPsychoJS, '', true);

// quit if user presses Cancel in dialog box:
dialogCancelScheduler.add(quitPsychoJS, '', false);

psychoJS.start({
  expName: expName,
  expInfo: expInfo,
  resources: [
    // resources:
    {'name': 'figures/prompt1.jpg', 'path': 'figures/prompt1.jpg'},
  ]
});

psychoJS.experimentLogger.setLevel(core.Logger.ServerLevel.EXP);


var currentLoop;
var frameDur;
async function updateInfo() {
  currentLoop = psychoJS.experiment;  // right now there are no loops
  expInfo['date'] = util.MonotonicClock.getDateStr();  // add a simple timestamp
  expInfo['expName'] = expName;
  expInfo['psychopyVersion'] = '2023.1.2';
  expInfo['OS'] = window.navigator.platform;


  // store frame rate of monitor if we can measure it successfully
  expInfo['frameRate'] = psychoJS.window.getActualFrameRate();
  if (typeof expInfo['frameRate'] !== 'undefined')
    frameDur = 1.0 / Math.round(expInfo['frameRate']);
  else
    frameDur = 1.0 / 60.0; // couldn't get a reliable measure so guess

  // add info from the URL:
  util.addInfoFromUrl(expInfo);
  

  
  psychoJS.experiment.dataFileName = (("." + "/") + `data/${expInfo["participant"]}_${expName}_${expInfo["date"]}`);


  return Scheduler.Event.NEXT;
}


var trial1Clock;
var prompt;
var Q;
var about_slider;
var image;
var slider;
var n_attempts;
var key_resp;
var globalClock;
var routineTimer;
async function experimentInit() {
  // Initialize components for Routine "trial1"
  trial1Clock = new util.Clock();
  prompt = new visual.ImageStim({
    win : psychoJS.window,
    name : 'prompt', units : undefined, 
    image : 'figures/prompt1.jpg', mask : undefined,
    anchor : 'center',
    ori : 0.0, pos : [(- 0.4), (- 0.08)], size : [0.5, 0.5],
    color : new util.Color([1,1,1]), opacity : undefined,
    flipHoriz : false, flipVert : false,
    texRes : 128.0, interpolate : true, depth : 0.0 
  });
  // Run 'Begin Experiment' code from rateCode
  psychoJS.window.color = 'black';
  psychoJS.window.flip();
  var slider_choices = [];
  var timestamps = [];
  Q = new visual.TextStim({
    win: psychoJS.window,
    name: 'Q',
    text: 'What is the minimum amount of food per day that the capybara needs to feel satiated?',
    font: 'Open Sans',
    units: undefined, 
    pos: [0, 0.4], height: 0.045,  wrapWidth: 1.0, ori: 0.0,
    languageStyle: 'LTR',
    color: new util.Color([1.0, 1.0, 1.0]),  opacity: undefined,
    depth: -2.0 
  });
  
  about_slider = new visual.TextStim({
    win: psychoJS.window,
    name: 'about_slider',
    text: 'Click on the slider below to find it out! Use the least amount of attempts you can.',
    font: 'Open Sans',
    units: undefined, 
    pos: [0, 0.29], height: 0.035,  wrapWidth: 2.0, ori: 0.0,
    languageStyle: 'LTR',
    color: new util.Color([1.0, 1.0, 1.0]),  opacity: undefined,
    depth: -3.0 
  });
  
  image = new visual.ImageStim({
    win : psychoJS.window,
    name : 'image', units : undefined, 
    image : undefined, mask : undefined,
    anchor : 'center',
    ori : 0, pos : [0.33, (- 0.11)], size : 1.0,
    color : new util.Color([1, 1, 1]), opacity : 1,
    flipHoriz : false, flipVert : false,
    texRes : 128, interpolate : true, depth : -4.0 
  });
  slider = new visual.Slider({
    win: psychoJS.window, name: 'slider',
    startValue: undefined,
    size: [0.6, 0.05], pos: [0.33, 0.17], ori: 0.0, units: psychoJS.window.units,
    labels: ["10", "20", "30", "40", "50", "60", "70", "80", "90", "100", "110", "120", "130"], fontSize: 0.02, ticks: [1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13],
    granularity: 1.0, style: ["SLIDER"],
    color: new util.Color([0.6549, 0.6549, 0.6549]), markerColor: new util.Color([1.0, (- 1.0), (- 1.0)]), lineColor: new util.Color([1.0, 1.0, 1.0]), 
    opacity: undefined, fontFamily: 'Open Sans', bold: true, italic: false, depth: -5, 
    flip: false,
  });
  
  n_attempts = new visual.TextStim({
    win: psychoJS.window,
    name: 'n_attempts',
    text: '',
    font: 'Open Sans',
    units: undefined, 
    pos: [0.33, (- 0.35)], height: 0.035,  wrapWidth: undefined, ori: 0.0,
    languageStyle: 'LTR',
    color: new util.Color([1.0, 1.0, 1.0]),  opacity: undefined,
    depth: -6.0 
  });
  
  key_resp = new core.Keyboard({psychoJS: psychoJS, clock: new util.Clock(), waitForStart: true});
  
  // Create some handy timers
  globalClock = new util.Clock();  // to track the time since experiment started
  routineTimer = new util.CountdownTimer();  // to track time remaining of each (non-slip) routine
  
  return Scheduler.Event.NEXT;
}


var t;
var frameN;
var continueRoutine;
var _key_resp_allKeys;
var trial1Components;
function trial1RoutineBegin(snapshot) {
  return async function () {
    TrialHandler.fromSnapshot(snapshot); // ensure that .thisN vals are up to date
    
    //--- Prepare to start Routine 'trial1' ---
    t = 0;
    trial1Clock.reset(); // clock
    frameN = -1;
    continueRoutine = true; // until we're told otherwise
    // update component parameters for each repeat
    // Run 'Begin Routine' code from rateCode
    //start rating
    var attempts = 0;
    var last_rating = null;
    var displayText = "";
    slider.reset()
    key_resp.keys = undefined;
    key_resp.rt = undefined;
    _key_resp_allKeys = [];
    // keep track of which components have finished
    trial1Components = [];
    trial1Components.push(prompt);
    trial1Components.push(Q);
    trial1Components.push(about_slider);
    trial1Components.push(image);
    trial1Components.push(slider);
    trial1Components.push(n_attempts);
    trial1Components.push(key_resp);
    
    for (const thisComponent of trial1Components)
      if ('status' in thisComponent)
        thisComponent.status = PsychoJS.Status.NOT_STARTED;
    return Scheduler.Event.NEXT;
  }
}


var last_rating;
var displayText;
function trial1RoutineEachFrame() {
  return async function () {
    //--- Loop for each frame of Routine 'trial1' ---
    // get current time
    t = trial1Clock.getTime();
    frameN = frameN + 1;// number of completed frames (so 0 is the first frame)
    // update/draw components on each frame
    
    // *prompt* updates
    if (t >= 0.0 && prompt.status === PsychoJS.Status.NOT_STARTED) {
      // keep track of start time/frame for later
      prompt.tStart = t;  // (not accounting for frame time here)
      prompt.frameNStart = frameN;  // exact frame index
      
      prompt.setAutoDraw(true);
    }

    // Run 'Each Frame' code from rateCode
    if (slider.getRating() === null) {  // Before any selection is made
        var imageName = 'figures/image0f.png';  // Show the initial image
        image.setImage(imageName);
    } else if (slider.getRating() !== null) {
        var imageIndex = Math.floor(slider.getRating()) - 1;
        var imageName = 'figures/image' + (imageIndex + 1) + 'f.png';
        image.setImage(imageName);
    }
    
    if (slider.getRating() !== null && slider.getRating() !== last_rating) {
        slider_choices.push(slider.getRating());
        timestamps.push(slider.getRT());
        attempts++;
    }
    last_rating = slider.getRating();  // Store the current rating to compare in the next frame
    displayText = "Number of attempts used: " + attempts;
    
    // *Q* updates
    if (t >= 0.0 && Q.status === PsychoJS.Status.NOT_STARTED) {
      // keep track of start time/frame for later
      Q.tStart = t;  // (not accounting for frame time here)
      Q.frameNStart = frameN;  // exact frame index
      
      Q.setAutoDraw(true);
    }

    
    // *about_slider* updates
    if (t >= 0.0 && about_slider.status === PsychoJS.Status.NOT_STARTED) {
      // keep track of start time/frame for later
      about_slider.tStart = t;  // (not accounting for frame time here)
      about_slider.frameNStart = frameN;  // exact frame index
      
      about_slider.setAutoDraw(true);
    }

    
    // *image* updates
    if (t >= 0.0 && image.status === PsychoJS.Status.NOT_STARTED) {
      // keep track of start time/frame for later
      image.tStart = t;  // (not accounting for frame time here)
      image.frameNStart = frameN;  // exact frame index
      
      image.setAutoDraw(true);
    }

    
    if (image.status === PsychoJS.Status.STARTED){ // only update if being drawn
      image.setSize([0.4, 0.4], false);
    }
    
    // *slider* updates
    if (t >= 0.0 && slider.status === PsychoJS.Status.NOT_STARTED) {
      // keep track of start time/frame for later
      slider.tStart = t;  // (not accounting for frame time here)
      slider.frameNStart = frameN;  // exact frame index
      
      slider.setAutoDraw(true);
    }

    
    // *n_attempts* updates
    if (t >= 0.0 && n_attempts.status === PsychoJS.Status.NOT_STARTED) {
      // keep track of start time/frame for later
      n_attempts.tStart = t;  // (not accounting for frame time here)
      n_attempts.frameNStart = frameN;  // exact frame index
      
      n_attempts.setAutoDraw(true);
    }

    
    if (n_attempts.status === PsychoJS.Status.STARTED){ // only update if being drawn
      n_attempts.setText(displayText, false);
    }
    
    // *key_resp* updates
    if (t >= 0.0 && key_resp.status === PsychoJS.Status.NOT_STARTED) {
      // keep track of start time/frame for later
      key_resp.tStart = t;  // (not accounting for frame time here)
      key_resp.frameNStart = frameN;  // exact frame index
      
      // keyboard checking is just starting
      psychoJS.window.callOnFlip(function() { key_resp.clock.reset(); });  // t=0 on next screen flip
      psychoJS.window.callOnFlip(function() { key_resp.start(); }); // start on screen flip
      psychoJS.window.callOnFlip(function() { key_resp.clearEvents(); });
    }

    if (key_resp.status === PsychoJS.Status.STARTED) {
      let theseKeys = key_resp.getKeys({keyList: ['e'], waitRelease: false});
      _key_resp_allKeys = _key_resp_allKeys.concat(theseKeys);
      if (_key_resp_allKeys.length > 0) {
        key_resp.keys = _key_resp_allKeys[_key_resp_allKeys.length - 1].name;  // just the last key pressed
        key_resp.rt = _key_resp_allKeys[_key_resp_allKeys.length - 1].rt;
        // a response ends the routine
        continueRoutine = false;
      }
    }
    
    // check for quit (typically the Esc key)
    if (psychoJS.experiment.experimentEnded || psychoJS.eventManager.getKeys({keyList:['escape']}).length > 0) {
      return quitPsychoJS('The [Escape] key was pressed. Goodbye!', false);
    }
    
    // check if the Routine should terminate
    if (!continueRoutine) {  // a component has requested a forced-end of Routine
      return Scheduler.Event.NEXT;
    }
    
    continueRoutine = false;  // reverts to True if at least one component still running
    for (const thisComponent of trial1Components)
      if ('status' in thisComponent && thisComponent.status !== PsychoJS.Status.FINISHED) {
        continueRoutine = true;
        break;
      }
    
    // refresh the screen if continuing
    if (continueRoutine) {
      return Scheduler.Event.FLIP_REPEAT;
    } else {
      return Scheduler.Event.NEXT;
    }
  };
}


function trial1RoutineEnd(snapshot) {
  return async function () {
    //--- Ending Routine 'trial1' ---
    for (const thisComponent of trial1Components) {
      if (typeof thisComponent.setAutoDraw === 'function') {
        thisComponent.setAutoDraw(false);
      }
    }
    // Run 'End Routine' code from rateCode
    for (var i = 0; i < slider_choices.length; i++) {
        psychoJS.experiment.addData('slider_choice', slider_choices[i]);
        psychoJS.experiment.addData('response_time', timestamps[i]);
        psychoJS.experiment.addData('attempts', attempts);
    }
    
    //psychoJS.experiment.addData('final_rating', slider.rating)
    psychoJS.experiment.addData('slider.response', slider.getRating());
    psychoJS.experiment.addData('slider.rt', slider.getRT());
    psychoJS.experiment.addData('slider.history', slider.getHistory());
    // update the trial handler
    if (currentLoop instanceof MultiStairHandler) {
      currentLoop.addResponse(key_resp.corr, level);
    }
    psychoJS.experiment.addData('key_resp.keys', key_resp.keys);
    if (typeof key_resp.keys !== 'undefined') {  // we had a response
        psychoJS.experiment.addData('key_resp.rt', key_resp.rt);
        routineTimer.reset();
        }
    
    key_resp.stop();
    // the Routine "trial1" was not non-slip safe, so reset the non-slip timer
    routineTimer.reset();
    
    // Routines running outside a loop should always advance the datafile row
    if (currentLoop === psychoJS.experiment) {
      psychoJS.experiment.nextEntry(snapshot);
    }
    return Scheduler.Event.NEXT;
  }
}


function importConditions(currentLoop) {
  return async function () {
    psychoJS.importAttributes(currentLoop.getCurrentTrial());
    return Scheduler.Event.NEXT;
    };
}


async function quitPsychoJS(message, isCompleted) {
  // Check for and save orphaned data
  if (psychoJS.experiment.isEntryEmpty()) {
    psychoJS.experiment.nextEntry();
  }
  
  
  psychoJS.window.close();
  psychoJS.quit({message: message, isCompleted: isCompleted});
  
  return Scheduler.Event.QUIT;
}
